/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataResponse;
import io.grpc.stub.StreamObserver;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

@Log4j2
@RequiredArgsConstructor(access = AccessLevel.PROTECTED)
class FileDownloadStreamObserver implements StreamObserver<GrpcGetBinaryFileDataResponse> {
	private final CompletableFuture<Boolean> streamFuture;
	private final OutputStream out;

	@Override
	public void onNext(GrpcGetBinaryFileDataResponse response) {
		writeToStream(response.getFileContent().toByteArray());
	}

	void writeToStream(byte[] contentPart) {
		try {
			out.write(contentPart);
		} catch (IOException e) {
			LOG.error("Download file error writing on output stream", e);
			throw new TechnicalException(e);
		}
	}

	@Override
	public void onError(Throwable t) {
		streamFuture.completeExceptionally(t);
	}

	@Override
	public void onCompleted() {
		streamFuture.complete(true);
	}
}
