/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Builder;

@Builder(toBuilder = true)
public record OzgFile(
  @Schema(description = "The id of the file, will be created by the OZG-Backend",
	name = "id",
	type = "string",
	example = "65248ad3f46811d04010f44bb"
  )
  String id,
  @Schema(description = "The name of the file. Optional when sending the reply",
	name = "fileName",
	type = "string",
	example = "TestDok.txt"
  )
  String fileName,
  @Schema(description = "The size if the file in bytes. Optional when sending the reply",
	name = "fileSize",
	type = "long",
	example = "236478552"
  )
  long fileSize,
  @Schema(description = "The content type of the file. Optional when sending the reply",
	name = "contentType",
	type = "string",
	example = "text/plain"
  )
  String contentType) {
}
