/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.command;

import org.springframework.stereotype.Component;

import de.ozgcloud.antragsraum.common.ChannelPropertiesFactory;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcGetCommandRequest;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.channelfactory.GrpcChannelFactory;
import net.devh.boot.grpc.client.interceptor.GlobalClientInterceptorRegistry;

@Log4j2
@Component
@RequiredArgsConstructor
public class CommandGrpcClient {
	private final @NonNull GlobalClientInterceptorRegistry registry;
	private final @NonNull ChannelPropertiesFactory channelPropertiesFactory;

	public GrpcCommand getCommand(GrpcGetCommandRequest request, String address) {
		try (GrpcChannelFactory channelFactory = getChannelFactory(address)) {
			var channel = channelFactory.createChannel(address);
			var commandServiceBlockingStub = CommandServiceGrpc.newBlockingStub(channel);

			return getGrpcCommand(commandServiceBlockingStub, request);
		}
	}

	GrpcCommand getGrpcCommand(CommandServiceGrpc.CommandServiceBlockingStub commandServiceBlockingStub, GrpcGetCommandRequest request) {
		return commandServiceBlockingStub.getCommand(request);
	}

	GrpcChannelFactory getChannelFactory(String channelAddress) {
		return de.ozgcloud.antragsraum.common.GrpcChannelFactory.getChannelFactory(channelPropertiesFactory.getGrpcChannelsProperties(channelAddress),
		  registry);
	}
}
