/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.command;

import java.time.ZonedDateTime;

import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class CommandMapper {
	static Command fromGrpcCommand(GrpcCommand grpcCommand, String nachrichtEventId) {
		var createdAt = ZonedDateTime.parse(grpcCommand.getCreatedAt()).toEpochSecond() * 1000;
		var finishedAt = grpcCommand.getFinishedAt().isEmpty() ? null : ZonedDateTime.parse(grpcCommand.getFinishedAt()).toEpochSecond() * 1000;

		return Command.builder()
		  .id(grpcCommand.getId())
		  .nachrichtEventId(nachrichtEventId)
		  .status(grpcCommand.getStatus())
		  .createdAt(createdAt)
		  .finishedAt(finishedAt)
		  .build();
	}
}
