/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.common;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import net.devh.boot.grpc.client.config.GrpcChannelProperties;
import net.devh.boot.grpc.client.config.GrpcChannelsProperties;
import net.devh.boot.grpc.client.config.NegotiationType;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

@Component
@RequiredArgsConstructor
public class ChannelPropertiesFactory {
    private final GrpcChannelsProperties grpcChannelsProperties;
    private final Map<String, GrpcChannelProperties> client = new ConcurrentHashMap<>();
    @Getter
    @Setter
    private String defaultScheme = "inProcess";

    @Value("${ozgcloud.grpc.client.negotiation-type:PLAINTEXT}")
    private String negotiationType;

    public GrpcChannelProperties getChannel(final String address) {
        GrpcChannelProperties properties = this.getRawChannel(address);
        properties.copyDefaultsFrom(grpcChannelsProperties.getGlobalChannel());
        return properties;
    }

    GrpcChannelProperties getRawChannel(final String nameAndAddress) {
        return this.client.computeIfAbsent(nameAndAddress, key -> createChannelProperties(nameAndAddress));
    }

    GrpcChannelProperties createChannelProperties(String nameAndAddress) {
        var globalChannel = grpcChannelsProperties.getGlobalChannel();
        GrpcChannelProperties prop = new GrpcChannelProperties();
        prop.copyDefaultsFrom(globalChannel);
        prop.setAddress(nameAndAddress);
        if (Objects.nonNull(negotiationType)) {
            prop.setNegotiationType(NegotiationType.valueOf(negotiationType));
        }
        return prop;
    }

    public GrpcChannelsProperties getGrpcChannelsProperties(String nameAndAddress) {
        var channel = getRawChannel(nameAndAddress);
        grpcChannelsProperties.getClient().put(nameAndAddress, channel);
        return grpcChannelsProperties;
    }
}
