/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import io.grpc.Metadata;
import io.grpc.ServerCall;
import io.grpc.ServerCallHandler;
import io.grpc.ServerInterceptor;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.interceptor.GrpcGlobalServerInterceptor;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.Objects;

@Log4j2
@GrpcGlobalServerInterceptor
@RequiredArgsConstructor
public class CallContextServerInterceptor implements ServerInterceptor {
	private final @NonNull JwtTokenWrapper tokenWrapper;

	@Override
	public <R, T> ServerCall.Listener<R> interceptCall(ServerCall<R, T> call, Metadata headers, ServerCallHandler<R, T> next) {
		var requestId = new String(Objects.requireNonNull(headers.get(ContextService.createKeyOf(ContextService.KEY_REQUEST_ID))));
		var jwtToken = new String(Objects.requireNonNull(headers.get(ContextService.createKeyOf(ContextService.KEY_JWT_TOKEN))));

		SecurityContextHolder.getContext().setAuthentication(tokenWrapper.wrap(jwtToken));

		LOG.info("Request Id {}; Set JWT token {}", requestId, jwtToken);
		return next.startCall(call, headers);
	}
}
