/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import java.util.UUID;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;

import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcGetCommandRequest;
import io.grpc.stub.StreamObserver;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@Log4j2
@ConditionalOnProperty(prefix = "grpc", name = "ozg_service.stubs.enabled")
public class CommandGrpcServiceStub extends CommandServiceGrpc.CommandServiceImplBase {
	static final String ID = UUID.randomUUID().toString();
	private static final String STATUS_FINISHED = "FINISHED";
	private static final String STATUS_PENDING = "PENDING";
	private static final String CREATED_AT = "2024-06-24T08:09:57.076Z";
	private static final String FINISHED_AT = "2024-06-24T08:10:13.076Z";

	@Override
	public void getCommand(GrpcGetCommandRequest request, StreamObserver<GrpcCommand> observer) {
		var grpcCommandBuilder = GrpcCommand.newBuilder()
		  .setId(ID)
		  .setCreatedAt(CREATED_AT);

		if (ID.equals(request.getId())) {
			LOG.info("Finishing command {}", request.getId());
			observer.onNext(grpcCommandBuilder.setStatus(STATUS_FINISHED).setFinishedAt(FINISHED_AT).build());
		} else {
			LOG.info("Pending command {}", request.getId());
			observer.onNext(grpcCommandBuilder.setStatus(STATUS_PENDING).setFinishedAt("").build());
		}
		observer.onCompleted();
	}
}
