/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import static de.ozgcloud.antragsraum.mocks.GrpcDataCreator.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Profile;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFile;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@RestController
@Log4j2
@RequestMapping(E2eTestController.PATH)
@RequiredArgsConstructor
@Profile("local")
public class E2eTestController {
	static final String CONTENT_TYPE = "text/plain";
	private static final String DATA = "Inhalt der Testdatei ";
	public static final String PATH = "/api/e2e/";

	private final StubDataRepository stubDataRepository;
	private final WritingNachrichtenEventService nachrichtEventService;

	@Operation(summary = "Create a Nachricht and send it to the InfoManager")
	@PostMapping(value = "event", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
	public ResponseEntity<NachrichtEvent> sendNachricht(@RequestBody @Parameter(description = "The Testevent data as JSON") MockEvent mockEvent) {
		var event = nachrichtEventService.addEvent(mockEvent);
		var nachrichtId = createNachricht(event, mockEvent);

		LOG.info("Created Nachricht Id: {}", nachrichtId);

		return ResponseEntity.ok().body(event);
	}

	String createNachricht(NachrichtEvent nachrichtEvent, MockEvent mockEvent) {
		var reply = GrpcDataCreator.getRueckfrageBuilder(mockEvent.vorgangId())
		  .setText(StringUtils.isNotEmpty(mockEvent.clerkMessageText()) ? mockEvent.clerkMessageText() : TEXT)
		  .setVorgangName(
			StringUtils.isNotEmpty(mockEvent.clerkMessageTitle()) ? mockEvent.clerkMessageTitle() : VORGANG_TITLE + mockEvent.vorgangId())
		  .setSentAt(StringUtils.isNotEmpty(mockEvent.sendDateTime())
			? mockEvent.sendDateTime()
			: LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));

		if (mockEvent.attachmentCount() > 0) {
			for (int i = 0; i < mockEvent.attachmentCount(); i++) {
				var fileId = UUID.randomUUID().toString();
				var fileContent = (DATA + i).getBytes();
				var file = GrpcBinaryFile.newBuilder()
				  .setId(fileId)
				  .setName("TestDoc" + i + ".txt")
				  .setContentType(CONTENT_TYPE)
				  .setSize(fileContent.length).build();
				reply.addAttachmentFileId(file.getId());

				stubDataRepository.setFile(file, reply.getVorgangId());
				stubDataRepository.addFileContent(fileId, fileContent);
			}
		}

		return String.valueOf(stubDataRepository.addReply(reply.build(), nachrichtEvent));
	}
}

