/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import java.nio.charset.StandardCharsets;

import org.apache.commons.lang3.RandomStringUtils;

import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageHead;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
@Log4j2
class GrpcDataCreator {
	static final String POSTFACH_ID_1 = "28721c6f-b78f-4d5c-a048-19fd2fc429d2";
	static final String NACHRICHT_ID_1 = "6358fd4146811d04010f44d0";
	static final String NACHRICHT_ID_2 = "6358fd4446811d04010f44da";
	static final String NACHRICHT_ID_3 = "6358fd4446811d04010f44dh";
	static final String VORGANG_ID_1 = "6358fd0bee7a051389cdd787";
	static final String VORGANG_ID_2 = "6358fd3f46811d04010f44c7";
	static final String VORGANG_ID_3 = "6358fd3f46811d04010f44c2";
	public static final String TEXT =
	  "Lorem ipsum dolor sit amet, consetetur sadipscing elitr,\nsed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat,\nsed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.\n\n"
		+ "Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.Lorem ipsum dolor sit amet, consetetur sadipscing elitr.";
	static final String FILE_ID_1 = "6358fd0bee7a051389cdd788";
	static final String FILE_NAME = "TestDokument.txt";
	static final byte[] FILE_CONTENT = TEXT.getBytes(StandardCharsets.UTF_8);
	static final String VORGANG_TITLE = "Vorgang title ";
	static final String SENT_AT = "2022-10-26T09:26:25";
	static final String ANSWERED_AT = "2022-11-26T09:26:25";
	static final String TRUST_LEVEL = "LOW";

	static GrpcRueckfrage.Builder getRueckfrageBuilder(String vorgangId) {
		var id = RandomStringUtils.randomAlphanumeric(16);
		var builder = GrpcRueckfrage.newBuilder()
		  .setId(id)
		  .setVorgangId(vorgangId)
		  .setText(TEXT)
		  .setVorgangName(VORGANG_TITLE + vorgangId)
		  .setSentAt(SENT_AT);

		if (vorgangId.equals(VORGANG_ID_1)) {
			builder
			  .setId(NACHRICHT_ID_1)
			  .addAttachmentFileId(FILE_ID_1)
			  .setAnsweredAt(ANSWERED_AT)
			  .setAccessible(true)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("ANSWERED")
			  .clearAnswers()
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().setRueckfrageId(id).setAnswerText("The Reply").build());
		} else if (vorgangId.equals(VORGANG_ID_2)) {
			builder
			  .setId(NACHRICHT_ID_2)
			  .setAccessible(true)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("NEW");
		} else {
			builder
			  .setId(NACHRICHT_ID_3)
			  .setAccessible(false)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("NEW");
		}

		return builder;
	}

	static GrpcRueckfrageHead.Builder getRueckfrageHeadBuilder(String vorgangId) {
		var id = RandomStringUtils.randomAlphanumeric(16);
		var builder = GrpcRueckfrageHead.newBuilder()
		  .setId(id)
		  .setVorgangId(vorgangId)
		  .setVorgangName(VORGANG_TITLE + vorgangId)
		  .setSentAt(SENT_AT);

		if (vorgangId.equals(VORGANG_ID_1)) {
			builder
			  .setId(NACHRICHT_ID_1)
			  .setAnsweredAt("2022-11-26T09:26:25")
			  .setAccessible(true)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("ANSWERED");
		} else if (vorgangId.equals(VORGANG_ID_2)) {
			builder
			  .setId(NACHRICHT_ID_2)
			  .setAccessible(true)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("NEW");
		} else {
			builder
			  .setId(NACHRICHT_ID_3)
			  .setAccessible(false)
			  .setTrustLevel(TRUST_LEVEL)
			  .setStatus("NEW");
		}

		return builder;
	}
}
