/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import java.io.IOException;
import java.util.UUID;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.DefaultRedirectStrategy;

import de.ozgcloud.antragsraum.security.InMemoryUserDetailService;
import de.ozgcloud.antragsraum.security.User;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class MockRedirectStrategy extends DefaultRedirectStrategy {
	private final UserDetailsService userDetailsService;

	public MockRedirectStrategy(UserDetailsService userDetailsService) {
		super();
		this.userDetailsService = userDetailsService;
	}

	private String onetimeToken;

	public void sendRedirect(HttpServletRequest request, HttpServletResponse response, String url, Authentication authentication) throws IOException {
		onetimeToken = UUID.randomUUID().toString();
		if (authentication instanceof UsernamePasswordAuthenticationToken userToken) {
			var authUser = (User) userToken.getPrincipal();
			var user = authUser.toBuilder()
			  .postkorbHandle(JwtTokenWrapper.POSTKORB_HANDLE)
			  .trustLevel(JwtTokenWrapper.USER_TRUST_LEVEL)
			  .refreshCode(RandomStringUtils.randomAlphanumeric(64))
			  .samlToken(JwtTokenWrapper.SAML_TOKEN_DUMMY).build();
			((InMemoryUserDetailService) userDetailsService).addUser(onetimeToken, user);
		} else {
			LOG.error("Invalid authentication received. Is {} expected {}", authentication.getClass().getName(),
			  UsernamePasswordAuthenticationToken.class.getName());
		}

		sendRedirect(request, response, url);
	}

	@Override
	public void sendRedirect(HttpServletRequest request, HttpServletResponse response, String url) throws IOException {
		String redirectUrl = this.calculateRedirectUrl(request.getContextPath(), url);
		redirectUrl = redirectUrl + "?code=" + onetimeToken;
		redirectUrl = response.encodeRedirectURL(redirectUrl);

		LOG.debug("Redirecting to {}", redirectUrl);

		response.sendRedirect(redirectUrl);
	}
}
