/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import java.util.Objects;
import java.util.UUID;

import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.security.User;
import de.ozgcloud.info.InformationServiceGrpc;
import de.ozgcloud.info.nachricht.GrpcNachricht;
import de.ozgcloud.info.nachricht.GrpcNewNachrichtRequest;
import de.ozgcloud.info.nachricht.NachrichtServiceGrpc;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
@Log4j2
public class WritingNachrichtenEventService {
	public static final String LOCALHOST_9090 = "static://localhost:9090";
	@GrpcClient("info-manager")
	private NachrichtServiceGrpc.NachrichtServiceBlockingStub nachrichtServiceBlockingStub;

	@GrpcClient("info-manager")
	private InformationServiceGrpc.InformationServiceBlockingStub informationServiceBlockingStub;

	public NachrichtEvent addEvent(MockEvent event) {
		var token = new UsernamePasswordAuthenticationToken(User.builder().samlToken("saml--token").build(), "dummy--token");
		SecurityContextHolder.getContext().setAuthentication(token);

		String listUrl = event.address() != null ? event.address() : LOCALHOST_9090;
		String postfachId = Objects.nonNull(event.postkorbHandle()) ? event.postkorbHandle() : UUID.randomUUID().toString();

		var nachricht = GrpcNachricht.newBuilder()
		  .setNachrichtenListUrl(listUrl)
		  .setPostfachId(postfachId)
		  .build();
		var res = nachrichtServiceBlockingStub.saveNewNachricht(GrpcNewNachrichtRequest.newBuilder()
		  .setNachricht(nachricht).build());

		LOG.info("Saved NachrichtEvent: {}, Result: {}", nachricht, res);
		return NachrichtEvent.builder()
		  .postfachId(postfachId)
		  .address("static://localhost:9091").build();
	}
}

