/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.time.DateTimeException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;

import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageHead;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@NoArgsConstructor(access = AccessLevel.PRIVATE)
class NachrichtHeaderMapper {
	static NachrichtHeader fromGrpcRueckfrageHead(GrpcRueckfrageHead grpcRueckfrageHead, String nachrichtEventId) {
		LOG.info("Mapping GrpcRueckfrageHead [{}] for NachrichtEvent [{}] to NachrichtHeader", grpcRueckfrageHead, nachrichtEventId);
		var builder = NachrichtHeader.builder()
		  .vorgangId(grpcRueckfrageHead.getVorgangId())
		  .id(grpcRueckfrageHead.getId())
		  .nachrichtEventId(nachrichtEventId)
		  .title(grpcRueckfrageHead.getVorgangName())
		  .answeredAt(toMillisecondsTimestamp(grpcRueckfrageHead.getAnsweredAt()))
		  .sendAt(toMillisecondsTimestamp(grpcRueckfrageHead.getSentAt()))
		  .accessible(grpcRueckfrageHead.getAccessible())
		  .trustLevel(TrustLevelMapper.map(grpcRueckfrageHead.getTrustLevel()))
		  .status(grpcRueckfrageHead.getStatus());

		return builder.build();
	}

	private static long toMillisecondsTimestamp(String dateString) {
		long timestamp = 0;
		try {
			timestamp = LocalDateTime.parse(dateString).toEpochSecond(ZoneOffset.UTC) * 1000;
		} catch (DateTimeException e) {
			LOG.warn("Invalid date [{}] received for Nachricht. Setting it to 0 (=1970-01-01)", dateString);
		}

		return timestamp;
	}
}
