/*
 * Copyright (c) 2024.   Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.InputStream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.protobuf.ByteString;

import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import io.grpc.stub.ClientCallStreamObserver;

@ExtendWith(MockitoExtension.class)
class ChunkedFileSenderTest {

	private ChunkedFileSender<GrpcUploadBinaryFileRequest> chunkedFileSender;

	@Spy
	InputStream inputStream = OzgUploadFileTestFactory.create().fileContent();

	@BeforeEach
	void setUp() {
		var metaDataRequest = GrpcUploadBinaryFileRequest.newBuilder().build();

		chunkedFileSender = spy(new ChunkedFileSender<>(inputStream, FileGrpcClient.CHUNK_SIZE,
		  this::buildChunkRequest, metaDataRequest));
	}

	GrpcUploadBinaryFileRequest buildChunkRequest(byte[] bytes) {
		return GrpcUploadBinaryFileRequest.newBuilder().setFileContent((ByteString.copyFrom(bytes))).build();
	}

	@Nested
	class TestSendingFileChunks {
		@Test
		@SuppressWarnings("unchecked")
		void shouldCallOnNext() {
			var observer = mock(ClientCallStreamObserver.class);

			chunkedFileSender.sendChunkTo(observer);

			verify(observer, atLeast(1)).onNext(any());
		}

		@Test
		@SuppressWarnings("unchecked")
		void shouldCallOnCompleted() {
			var observer = mock(ClientCallStreamObserver.class);

			chunkedFileSender.sendChunkTo(observer);

			verify(observer).onCompleted();
		}

		@Test
		@SuppressWarnings("unchecked")
		void shouldCallReadNBytes() throws IOException {
			var observer = mock(ClientCallStreamObserver.class);

			chunkedFileSender.sendChunkTo(observer);

			verify(inputStream).readNBytes(anyInt());
		}

		@Test
		@SuppressWarnings("unchecked")
		void shouldCallInputStreamClose() throws IOException {
			var observer = mock(ClientCallStreamObserver.class);

			chunkedFileSender.sendChunkTo(observer);

			verify(inputStream).close();
		}
	}
}