/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.protobuf.ByteString;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataResponse;

@ExtendWith(MockitoExtension.class)
class FileDownloadStreamObserverTest {
	@InjectMocks
	private FileDownloadStreamObserver downloadStreamObserver;
	@Mock
	private CompletableFuture<Boolean> streamFuture;
	@Mock
	private OutputStream out;

	@Nested
	class TestOnNext {
		@Test
		void shouldWriteToStreamOnReceivingContentPart() throws IOException {
			downloadStreamObserver.onNext(createFileContentResponse());

			verify(out).write(DATA);
		}

		private GrpcGetBinaryFileDataResponse createFileContentResponse() {
			return GrpcGetBinaryFileDataResponse.newBuilder().setFileContent(ByteString.copyFrom(DATA)).build();
		}

		@Test
		void shouldHandleException() throws IOException {
			doThrow(IOException.class).when(out).write(any());

			assertThatExceptionOfType(TechnicalException.class).isThrownBy(
			  () -> downloadStreamObserver.onNext(createFileContentResponse())).withMessage("TechnicalException happened! Message: null");
		}
	}

	@Nested
	class TestOnError {
		@Mock
		private Throwable throwable;

		@Test
		void shouldCompleteFutureExceptionally() {
			downloadStreamObserver.onError(throwable);

			verify(streamFuture).completeExceptionally(throwable);
		}
	}

	@Nested
	class TestOnCompleted {
		@Test
		void shouldCompleteFuture() {
			downloadStreamObserver.onCompleted();

			verify(streamFuture).complete(true);
		}
	}
}