/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;

import com.google.protobuf.ByteString;

import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataResponse;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileResponse;
import de.ozgcloud.vorgang.grpc.file.GrpcOzgFile;
import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@Log4j2
public class GrpcBinaryFileServiceStub extends BinaryFileServiceGrpc.BinaryFileServiceImplBase {
	public void getBinaryFileContent(GrpcGetBinaryFileDataRequest request,
	  StreamObserver<GrpcGetBinaryFileDataResponse> responseObserver) {
		responseObserver.onNext(
		  GrpcGetBinaryFileDataResponse.newBuilder().setFileContent(ByteString.copyFrom(DATA))
			.build());
		responseObserver.onCompleted();
	}

	public void findBinaryFilesMetaData(GrpcBinaryFilesRequest request,
	  StreamObserver<GrpcFindFilesResponse> responseObserver) {
		var file = GrpcOzgFile.newBuilder()
		  .setContentType(CONTENT_TYPE)
		  .setSize(FILE_SIZE)
		  .setId(FILE_ID)
		  .setName(FILE_NAME)
		  .build();

		responseObserver.onNext(GrpcFindFilesResponse.newBuilder().addFile(file).build());
		responseObserver.onCompleted();
	}

	public StreamObserver<GrpcUploadBinaryFileRequest> uploadBinaryFileAsStream(StreamObserver<GrpcUploadBinaryFileResponse> responseObserver) {
		return new CallStreamObserver<>() {

			@Override
			public boolean isReady() {
				return true;
			}

			@Override
			public void setOnReadyHandler(final Runnable onReadyHandler) {
			}

			@Override
			public void disableAutoInboundFlowControl() {
				// ignored
			}

			@Override
			public void request(final int count) {
				// ignore
			}

			@Override
			public void setMessageCompression(final boolean enable) {
				// ignored
			}

			@Override
			public void onNext(final GrpcUploadBinaryFileRequest value) {
				LOG.info("onNext called");
			}

			@Override
			public void onError(final Throwable t) {
				responseObserver.onError(t);
			}

			@Override
			public void onCompleted() {
				responseObserver.onNext(GrpcUploadBinaryFileResponse.newBuilder().setFileId(FILE_ID).build());
				responseObserver.onCompleted();
			}
		};
	}
}
