/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.callcontext;

import de.ozgcloud.antragsraum.callcontext.CallContextAttachingInterceptor.CallContextAttachingClientCall;
import io.grpc.Channel;
import io.grpc.ClientCall;
import io.grpc.Metadata;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class CallContextAttachingInterceptorTest<A, B> {
    @InjectMocks
    private CallContextAttachingInterceptor interceptor;
    @Mock
    private ContextService contextService;

    private CallContextAttachingClientCall<A, B> clientCall;
    @Mock
    private ClientCall<A, B> delegateCall;

    @Mock
    private ClientCall.Listener<B> responseListener;

    @Mock
    private Metadata contextMetadata;
    @Mock
    private Metadata inputMetadata;

    @BeforeEach
    void initClientCall() {
        clientCall = interceptor.new CallContextAttachingClientCall<>(delegateCall);
    }

    @Nested
    class TestInterceptorMethods {
        @BeforeEach
        void mockContextService() {
            when(contextService.buildCallContextMetadata()).thenReturn(contextMetadata);
        }

        @Test
        void shouldCallUserService() {
            clientCall.start(responseListener, inputMetadata);

            verify(contextService).buildCallContextMetadata();
        }

        @Test
        void shouldMergeMetas() {
            clientCall.start(responseListener, inputMetadata);

            verify(inputMetadata).merge(contextMetadata);
        }
    }


    @Test
    void shouldInterceptCall() {
        var res = interceptor.interceptCall(null, null, mock(Channel.class));

        assertThat(res).isNotNull();
    }
}