/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.common.collect.ImmutableList;

import de.ozgcloud.antragsraum.common.ChannelPropertiesFactory;
import de.ozgcloud.vorgang.grpc.command.CommandServiceGrpc;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcGetCommandRequest;
import io.grpc.Channel;
import net.devh.boot.grpc.client.channelfactory.GrpcChannelFactory;
import net.devh.boot.grpc.client.config.GrpcChannelsProperties;
import net.devh.boot.grpc.client.interceptor.GlobalClientInterceptorRegistry;

@ExtendWith(MockitoExtension.class)
class CommandGrpcClientTest {
	private final static String CHANNEL_ADDRESS = "static://localhost:9090";

	@Spy
	@InjectMocks
	private CommandGrpcClient grpcClient;

	@Mock
	private GlobalClientInterceptorRegistry registry;

	@Mock
	private ChannelPropertiesFactory channelPropertiesFactory;

	@Nested
	class TestCreatingChannel {
		@BeforeEach
		void init() {
			when(channelPropertiesFactory.getGrpcChannelsProperties(anyString())).thenReturn(new GrpcChannelsProperties());
		}

		@Test
		void shouldGetChannelFactory() {
			var channelFactory = grpcClient.getChannelFactory(CHANNEL_ADDRESS);

			assertThat(channelFactory).isNotNull();
		}

		@Test
		void shouldGetChannel() {
			when(registry.getClientInterceptors()).thenReturn(ImmutableList.of());

			var channel = grpcClient.getChannelFactory(CHANNEL_ADDRESS).createChannel(CHANNEL_ADDRESS);

			assertThat(channel).isNotNull();
		}
	}

	@Nested
	class TestGetCommand {
		@Mock
		private CommandServiceGrpc.CommandServiceBlockingStub commandServiceBlockingStub;
		@Mock
		private GrpcGetCommandRequest request;

		@Test
		void shouldCallGetGrpcCommand() {
			var channelFactory = mock(GrpcChannelFactory.class);
			var channel = mock(Channel.class);
			when(channelFactory.createChannel(anyString())).thenReturn(channel);
			doReturn(channelFactory).when(grpcClient).getChannelFactory(anyString());
			doReturn(mock(GrpcCommand.class)).when(grpcClient).getGrpcCommand(any(), any(GrpcGetCommandRequest.class));

			var request = GrpcGetCommandRequest.newBuilder().setId(UUID.randomUUID().toString()).build();
			grpcClient.getCommand(request, CHANNEL_ADDRESS);

			verify(grpcClient).getGrpcCommand(any(), any(GrpcGetCommandRequest.class));
		}

		@Test
		void shouldGetCommand() {
			grpcClient.getGrpcCommand(commandServiceBlockingStub, request);

			verify(commandServiceBlockingStub).getCommand(any(GrpcGetCommandRequest.class));
		}
	}
}