/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.command;

import java.time.ZonedDateTime;
import java.util.UUID;

import de.ozgcloud.vorgang.grpc.command.GrpcCommand;

public class CommandTestFactory {
	public static final String ID = UUID.randomUUID().toString();
	static final String NACHRICHT_EVENT_ID = UUID.randomUUID().toString();
	static final String STATUS = "FINISHED";
	static final String CREATED_AT = "2024-06-24T08:09:57.076Z";
	static final long CREATED_AT_DATE = ZonedDateTime.parse(CREATED_AT).toEpochSecond() * 1000;
	static final String FINISHED_AT = "2024-06-24T08:10:13.076Z";
	static final long FINISHED_AT_DATE = ZonedDateTime.parse(FINISHED_AT).toEpochSecond() * 1000;

	static Command createCommand() {
		return createCommandBuilder().build();
	}

	static Command.CommandBuilder createCommandBuilder() {
		return Command.builder()
		  .id(ID)
		  .nachrichtEventId(NACHRICHT_EVENT_ID)
		  .status(STATUS)
		  .createdAt(CREATED_AT_DATE)
		  .finishedAt(FINISHED_AT_DATE);
	}

	static GrpcCommand createGrpcCommand() {
		return createGrpcCommandBuilder().build();
	}

	static GrpcCommand.Builder createGrpcCommandBuilder() {
		return GrpcCommand.newBuilder()
		  .setId(ID)
		  .setStatus(STATUS)
		  .setCreatedAt(CREATED_AT)
		  .setFinishedAt(FINISHED_AT);
	}
}
