/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.events;

import java.util.UUID;

import de.ozgcloud.info.GrpcInformationByIdRequest;
import de.ozgcloud.info.GrpcInformationByIdResponse;
import de.ozgcloud.info.GrpcInformationNachricht;
import de.ozgcloud.info.GrpcInformationRequest;
import de.ozgcloud.info.GrpcInformationResponse;
import de.ozgcloud.info.InformationServiceGrpc;
import io.grpc.Status;
import io.grpc.stub.StreamObserver;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
public class InformationGrpcServiceStub extends InformationServiceGrpc.InformationServiceImplBase {
	public static final String ID = UUID.randomUUID().toString();
	public static final String EMPTY_ID = UUID.randomUUID().toString();
	public static final String POSTFACH_ID = UUID.randomUUID().toString();
	public static final String EMPTY_POSTFACH_ID = "550e8400-e29b-41d4-a716-446655440000";

	@Override
	public void getInformationById(final GrpcInformationByIdRequest request, final StreamObserver<GrpcInformationByIdResponse> responseObserver) {
		if (request.getId().equals(EMPTY_ID)) {
			responseObserver.onError(Status.INVALID_ARGUMENT.asRuntimeException());
		} else {
			responseObserver.onNext(GrpcInformationByIdResponse.newBuilder()
			  .setNachricht(GrpcInformationNachricht.newBuilder()
				.setPostfachId(POSTFACH_ID)
				.build())
			  .build());
			responseObserver.onCompleted();
		}
	}

	@Override
	public void getInformation(GrpcInformationRequest request, StreamObserver<GrpcInformationResponse> responseObserver) {
		if (request.getPostfachId().equals(EMPTY_POSTFACH_ID)) {
			responseObserver.onNext(GrpcInformationResponse.getDefaultInstance());
		} else {
			responseObserver.onNext(GrpcInformationResponse.newBuilder()
			  .addNachrichten(GrpcInformationNachricht.newBuilder()
				.setPostfachId(POSTFACH_ID)
				.build()).build()
			);
		}

		responseObserver.onCompleted();
	}
}
