/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.events;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.info.GrpcInformationNachricht;

class NachrichtEventMapperTest {
	@Nested
	class TestFromGrpcInformationNachricht {
		private final GrpcInformationNachricht grpcInformationNachricht = NachrichtEventTestFactory.createGrpcInformationNachricht();

		@Test
		void shouldMap() {
			var nachrichtEvent = NachrichtEventMapper.fromGrpcInformationNachricht(grpcInformationNachricht);

			assertThat(nachrichtEvent).isNotNull();
		}

		@Test
		void shouldHaveId() {
			var nachrichtEvent = NachrichtEventMapper.fromGrpcInformationNachricht(grpcInformationNachricht);

			assertThat(nachrichtEvent.id()).isEqualTo(NachrichtEventTestFactory.ID);
		}

		@Test
		void shouldHavePostfachId() {
			var nachrichtEvent = NachrichtEventMapper.fromGrpcInformationNachricht(grpcInformationNachricht);

			assertThat(nachrichtEvent.postfachId()).isEqualTo(NachrichtEventTestFactory.POSTFACH_ID);
		}

		@Test
		void shouldHaveOzgCloudAddress() {
			var nachrichtEvent = NachrichtEventMapper.fromGrpcInformationNachricht(grpcInformationNachricht);

			assertThat(nachrichtEvent.address()).isEqualTo(NachrichtEventTestFactory.URL);
		}
	}
}