/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.events;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;

import io.grpc.StatusRuntimeException;

@SpringBootTest(properties = {
  "grpc.server.inProcessName=test",
  "grpc.server.port=-1",
  "grpc.client.inProcess.address=in-process:test"
})
@SpringJUnitConfig(classes = { NachrichtEventGrpcTestConfiguration.class })
@DirtiesContext
@ExtendWith(MockitoExtension.class)
class NachrichtEventRemoteServiceITCase {
	@Autowired
	NachrichtEventRemoteService nachrichtEventRemoteService;

	@Nested
	class TestInformationAvailable {
		@Test
		@DirtiesContext
		void shouldLoadNachrichtEvents() {
			var result = nachrichtEventRemoteService.getNachrichtEventsOfPostfach(InformationGrpcServiceStub.POSTFACH_ID);

			assertThat(result).hasSize(1);
		}
	}

	@Nested
	class TestNoInformationAvailable {
		@Test
		void shouldLoadEmptyNachrichtEvents() {
			var result = nachrichtEventRemoteService.getNachrichtEventsOfPostfach(InformationGrpcServiceStub.EMPTY_POSTFACH_ID);

			assertThat(result).isEmpty();
		}
	}

	@Nested
	class TestInformationByIdAvailable {
		@Test
		@DirtiesContext
		void shouldLoadNachrichtEventById() {
			var result = nachrichtEventRemoteService.getNachrichtEventById(InformationGrpcServiceStub.ID);

			assertThat(result).isInstanceOf(NachrichtEvent.class);
		}
	}

	@Nested
	class TestNoInformationByIdAvailable {
		@Test
		@DirtiesContext
		void shouldThrowStatusRuntimeException() {
			assertThrows(StatusRuntimeException.class, () -> nachrichtEventRemoteService.getNachrichtEventById(InformationGrpcServiceStub.EMPTY_ID));
		}
	}
}