/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.List;
import java.util.UUID;

import de.ozgcloud.antragsraum.command.CommandTestFactory;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcGetRueckfrageResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageHead;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerResponse;

public class GrpcRueckfrageTestFactory {
	public static final String RUECKFRAGE_ID = UUID.randomUUID().toString();
	public static final String VORGANG_ID = UUID.randomUUID().toString();
	public static final String VORGANG_NAME = NachrichtTestFactory.VORGANG_TITLE;
	public static final String ANSWER_FILE_ID = UUID.randomUUID().toString();
	public static final String TEXT = """
	  Lorem ipsum dolor sit amet, consetetur sadipscing elitr,
	   sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat,
	   sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum.
	   
	   Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.
	   Lorem ipsum dolor sit amet, consetetur sadipscing elitr.
	  """;
	public static final String ANSWER_TEXT = """
	  ANTWORT Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.
	   Lorem ipsum dolor sit amet, consetetur sadipscing elitr.
	  """;
	public static final String SEND_AT = "2022-10-26T09:26:25";
	public static final String SEND_AT_LONG = "2022-10-26T09:26:25.242804292Z[UTC]";
	public static final long SEND_AT_DATE = LocalDateTime.parse(SEND_AT).toEpochSecond(ZoneOffset.UTC) * 1000;
	public static final String ANSWERED_AT = "2022-10-27T12:00:00";
	public static final long ANSWERED_AT_DATE = LocalDateTime.parse(ANSWERED_AT).toEpochSecond(ZoneOffset.UTC) * 1000;
	public static final boolean ACCESSIBLE = true;
	public static final String TRUST_LEVEL = "LOW";
	public static final String STATUS = "ANSWERED";

	static GrpcGetRueckfrageResponse createGetRueckfrageResponse() {
		return GrpcGetRueckfrageResponse.newBuilder().setRueckfrage(createRueckfrage()).build();
	}

	static GrpcRueckfrage createRueckfrage() {
		return createRueckfrageBuilder().build();
	}

	static GrpcRueckfrage createRueckfrageWithReply() {
		return createRueckfrageBuilder()
		  .setAnsweredAt(ANSWERED_AT)
		  .setSentAt(SEND_AT)
		  .addAnswers(GrpcRueckfrageAnswer.newBuilder()
			.setAnswerText(ANSWER_TEXT)
			.setSentAt(SEND_AT)
			.addAttachmentFileId(ANSWER_FILE_ID)
			.build())
		  .build();
	}

	static GrpcRueckfrage.Builder createRueckfrageBuilder() {
		return GrpcRueckfrage.newBuilder()
		  .setId(RUECKFRAGE_ID)
		  .setVorgangId(VORGANG_ID)
		  .setVorgangName(VORGANG_NAME)
		  .setText(TEXT)
		  .setSentAt(SEND_AT)
		  .setAccessible(ACCESSIBLE)
		  .setTrustLevel(TRUST_LEVEL)
		  .setStatus(STATUS)
		  .addAllAttachmentFileId(
			List.of(NachrichtTestFactory.ATTACHMENT_ID_1, NachrichtTestFactory.ATTACHMENT_ID_2));
	}

	static GrpcFindRueckfragenResponse createFindRueckfragenResponse() {
		return GrpcFindRueckfragenResponse.newBuilder().addRueckfrageHead(createRueckfrageHead()).build();
	}

	static GrpcRueckfrageHead createRueckfrageHead() {
		return createRueckfrageHeadBuilder().build();
	}

	static GrpcRueckfrageHead.Builder createRueckfrageHeadBuilder() {
		return GrpcRueckfrageHead.newBuilder()
		  .setId(RUECKFRAGE_ID)
		  .setVorgangId(VORGANG_ID)
		  .setVorgangName(VORGANG_NAME)
		  .setSentAt(SEND_AT)
		  .setAnsweredAt(ANSWERED_AT)
		  .setAccessible(ACCESSIBLE)
		  .setTrustLevel(TRUST_LEVEL)
		  .setStatus(STATUS);
	}

	static GrpcSendRueckfrageAnswerResponse createSendRueckfrageAnswerResponse() {
		return GrpcSendRueckfrageAnswerResponse.newBuilder().setCommandId(CommandTestFactory.ID).build();
	}
}
