/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageHead;

class NachrichtHeaderMapperTest {
	private final GrpcRueckfrageHead rueckfrageHead = GrpcRueckfrageTestFactory.createRueckfrageHead();

	@Test
	void shouldMapId() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.id()).isEqualTo(GrpcRueckfrageTestFactory.RUECKFRAGE_ID);
	}

	@Test
	void shouldMapNachrichtEventId() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.nachrichtEventId()).isEqualTo(NachrichtTestFactory.NACHRICHT_EVENT_ID);
	}

	@Test
	void shouldMapVorgangId() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.vorgangId()).isEqualTo(GrpcRueckfrageTestFactory.VORGANG_ID);
	}

	@Test
	void shouldMapVorgangName() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.title()).isEqualTo(GrpcRueckfrageTestFactory.VORGANG_NAME);
	}

	@Test
	void shouldMapAnsweredAtDate() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.answeredAt()).isEqualTo(GrpcRueckfrageTestFactory.ANSWERED_AT_DATE);
	}

	@Test
	void shouldMapEmptyAnsweredAtDate() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(
		  GrpcRueckfrageTestFactory.createRueckfrageHeadBuilder().setAnsweredAt("").build(), NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.answeredAt()).isEqualTo(0);
	}

	@Test
	void shouldMapSendAtDate() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.sendAt()).isEqualTo(GrpcRueckfrageTestFactory.SEND_AT_DATE);
	}

	@Test
	void shouldMapAccessible() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.accessible()).isEqualTo(GrpcRueckfrageTestFactory.ACCESSIBLE);
	}

	@Test
	void shouldMapTrustLevel() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.trustLevel()).isEqualTo(GrpcRueckfrageTestFactory.TRUST_LEVEL);
	}

	@Test
	void shouldMapStatus() {
		var nachrichtHeader = NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, NachrichtTestFactory.NACHRICHT_EVENT_ID);

		assertThat(nachrichtHeader.status()).isEqualTo(GrpcRueckfrageTestFactory.STATUS);
	}
}